#ifndef DISPLAY_H
#define DISPLAY_H

#include "flash.h"
#include "string.h"

namespace sys
{

class Display
{
public:
	static char display[128];

private:
	int where;
	int line;

public:
	Display();
	~Display();

	int blank(int n=0, char c = ' ');
	int blankDisplay(char c = ' ');

	int showDisplay(int whichflash);

	void put (char c);
	void nput (char c);

	void show_number (int i, int anz);
	int get_anz (int i);
};

Display::Display() :
	where (0),
	line (0)
{
	blankDisplay();
}

Display::~Display()
{
}

/**puts n spaces to display.

  where is set to the position afterwards.
  */
int Display::blank(int n, char c)
{
	for (int i=0; i!=n; i++)
	{
		put(c);
	}
}

int Display::blankDisplay(char c)
{
	for (int i=0; i!=128; i++)
	{
		display[i] = c;
	}
}

int Display::showDisplay(int whichflash)
{
	Flash f;
	f.read(whichflash);
	strncpy((char*)display, (char*)f.flash, 128);
}

void Display::put (char c)
{
	display[where] = c;
	where ++;
	if (where == 32) line++;
	else if (where == 64) line++;
	else if (where == 96) line++;
	else if (where == 128)
	{
		where = 0;
		line = 0;
	}
}

void Display::nput (char c)
{
	display[where] = c;
	where --;
	if (where == 32) line++;
	else if (where == 64) line++;
	else if (where == 96) line++;
	else if (where == 128)
	{
		where = 0;
		line = 0;
	}
}


void Display::show_number(int i, int anz)
{
	int m = 0; // the current modulus of the number
	int x = 10; // basis
	int z = '0'; // ascii value of the digit 0
	where = add(anz, 4);

	while (i)
	{
		m=mod(i,x);
		i=div(i,x);
		m=add(m,z);

		nput(m);
	}

}

int Display::get_anz(int i)
{
	int m = 0; // the current modulus of the number
	int x = 10; // basis
	int z = '0'; // ascii value of the digit 0
	int anz = 0;

	while (i!=0)
	{
		m=mod(i,x);
		i=div(i,x);
		m=add(m,z);
		anz++;

		put(m);
	}

	return anz;
}

Display& operator << (Display& d, char c)
{
	d.put(c);
}


char Display::display[128];

}

#endif
