#include "flash.h"
#include "display.h"
#include "input.h"

#include "storage.h"

extern "C"
{
int __set(int reg, int val);
int __get(int reg);
} // end extern "C"

namespace editor
{

int set (int reg, int val)
{
	return __set(reg, val);
}

int get (int reg)
{
	return __get(reg);
}

int input_number()
{
	using namespace sys;
	Display d;
	d.showDisplay(storage::enter_number);
	return sys::read_number();
}

int input_char()
{
	using namespace sys;
	Display d;
	d.showDisplay(storage::enter_character);
	int op = sys::getchar();
	int ig = sys::getchar();

	return op;
}

int flash_read()
{
	int nr = input_number();

	sys::Display d;
	d.showDisplay(nr);

	sys::getchar();
}

int flash_write_info()
{
	sys::Display d;
	d.showDisplay(storage::enter_flash);
	sys::getchar();
}


int flash_write()
{
	flash_write_info();

	int block = input_number();
	int where = input_number();
	int newchar = input_char();

	sys::Flash f;

	f.read(block);
	f.change(where, newchar);
	f.write(block);

	// Immediately apply
}

int reg_read()
{
	using namespace sys;
	int nr = input_number();

	int reg = get(nr);

	Display d;
	d << (char) reg;
	sys::getchar();
}

int reg_write()
{
	using namespace sys;
	int nr = input_number();
	int c = input_char();

	set(nr,c);
}

int main()
{
	using namespace sys;

	while (true)
	{
		Display d;
		d.showDisplay(storage::editor);
		int todo = sys::getchar();
		int ignore = sys::getchar();

		if (todo == 'F') flash_read();
		else if (todo == 'R') reg_read();
		if (todo == 'L') flash_write();
		else if (todo == 'E') reg_write();
		else if (todo == 'V') return 0;
	}

	return 0;
}

} // end namespace editor
