#include <fstream>
#include <iostream>
#include <string>

#include "opcode.h"

#include "config.h"

#include "getopt_pp.h"

using namespace GetOpt;
using namespace std;

void read_file(Opcode &opcodes, istream &file)
{
	int linenr = 0;
	string op, A, B;
	while (file >> op >> B >> A)
	{
		linenr++;

		// cout << "read: " << op << " " << A << " " << B << endl;
		try {
			opcodes.add (op, A, B);
		} catch (invalid& i)
		{
			i.linenr = linenr;
			throw;
		}
	}
}

void help()
{
	cout << "basicasm" << endl;
	cout << "Assembles the basic assembler syntax to a binary" << endl;
	cout << "format the simulator understands" << endl;
	cout << endl;
	cout << "It reads in the triple:" << endl;
	cout << "opcode iB iA" << endl;
	cout << "where all three can be numbers or symbolic constants." << endl;
	cout << "For supported commands and registers, see opcode.cpp" << endl;
	cout << "oBA is written out in a binary flash file." << endl;
	cout << endl;
	cout << "  -i infile (in.b)" << endl;
	cout << "  -o outfile (out.f)" << endl;
}

int main(int argc, char ** argv) try
{
	string infile;
	string outfile;

	GetOpt_pp options (argc, argv);
	options.exceptions (ios::failbit | ios::eofbit);

	options >> Option ('i', "infile", infile, "in.b")
		>> Option ('o', "outfile", outfile, "out.f");

	cout << "Basicasm reading from " << infile
		<< " and writing opcodes to " << outfile << endl;

	Opcode opcodes;

	ifstream fin (infile.c_str());
	if (!fin.is_open())
	{
		cerr << "Could not open file "
			<< infile << endl;
		help();
		return 1;
	}

	read_file(opcodes, fin);

	ofstream fout (outfile.c_str());
	if (!fout.is_open())
	{
		cerr << "Could not open file "
			<< outfile << endl;
		help();
		return 1;
	}


	fout.write (opcodes.buffer(), opcodes.size());

} catch (GetOptEx const& e)
{
	cerr << "Invalid Options" << endl;
	help();
	return 1;
} catch (invalid_opcode const& i)
{
	cerr << "You have used an invalid opcode: " << i.what() << endl;
	cerr << "Occurred at line: " << i.linenr << endl;
	return 2;
}
catch (invalid_register const& i)
{
	cerr << "You have used an invalid register: " << i.what() << endl;
	cerr << "Occurred at line: " << i.linenr << endl;
	cerr << "The problem is at register: " << i.reg << endl;
	return 3;
}
catch (...)
{
	cerr << "Caught unknown exception" << endl;
	return 100;
}

