#include "opcode.h"

#include <sstream>
#include <iostream>

using namespace std;

Opcode::Opcode() :
	opcodes(numreg),
	commands(),
	registers(),
	where(0)
{
	commands.insert(pair<string, word_t> ("setc", 0));
	commands.insert(pair<string, word_t> ("set", 1));
	commands.insert(pair<string, word_t> ("setac", 2));
	commands.insert(pair<string, word_t> ("seta", 3));
	commands.insert(pair<string, word_t> ("addc", 4));
	commands.insert(pair<string, word_t> ("add", 5));
	commands.insert(pair<string, word_t> ("adda", 6));
	commands.insert(pair<string, word_t> ("sub", 7));
	commands.insert(pair<string, word_t> ("mulc", 8));
	commands.insert(pair<string, word_t> ("mul", 9));
	commands.insert(pair<string, word_t> ("mod", 10)); // a
	commands.insert(pair<string, word_t> ("div", 11)); // b
	commands.insert(pair<string, word_t> ("jump", 12)); // c
	commands.insert(pair<string, word_t> ("key", 13)); // d
	commands.insert(pair<string, word_t> ("write", 14)); // e
	commands.insert(pair<string, word_t> ("read", 15)); // f

	registers.insert(pair<string, word_t> ("lowaddr", lowaddr));
	registers.insert(pair<string, word_t> ("highaddr", highaddr));

	registers.insert(pair<string, word_t> ("lowvar", lowvar));
	registers.insert(pair<string, word_t> ("highvar", highvar));

	registers.insert(pair<string, word_t> ("lowflash", lowflash));
	registers.insert(pair<string, word_t> ("highflash", highflash));

	registers.insert(pair<string, word_t> ("pc", pc));
	registers.insert(pair<string, word_t> ("PC", pc));

	registers.insert(pair<string, word_t> ("start", start)); // the start address, main will be called there
	registers.insert(pair<string, word_t> ("neg", pc+1)); // always neg (-1) in there
	registers.insert(pair<string, word_t> ("zero", pc+2)); // always zero (0) in there
	registers.insert(pair<string, word_t> ("one", pc+3)); // always one (1) in there
	registers.insert(pair<string, word_t> ("two", pc+4)); // always two (2) in there

	registers.insert(pair<string, word_t> ("Z", Z)); // compare register
	registers.insert(pair<string, word_t> ("z", Z));

	registers.insert(pair<string, word_t> ("W", W)); // general working register
	registers.insert(pair<string, word_t> ("w", W));

	registers.insert(pair<string, word_t> ("W0", pc+7)); // specific working registers
	registers.insert(pair<string, word_t> ("W1", pc+8));
	registers.insert(pair<string, word_t> ("W2", pc+9));
	registers.insert(pair<string, word_t> ("W3", pc+10));
	registers.insert(pair<string, word_t> ("W4", pc+11));
	registers.insert(pair<string, word_t> ("W5", pc+12));
	registers.insert(pair<string, word_t> ("W6", pc+13));
	registers.insert(pair<string, word_t> ("W7", pc+14));

	registers.insert(pair<string, word_t> ("w0", pc+7));
	registers.insert(pair<string, word_t> ("w1", pc+8));
	registers.insert(pair<string, word_t> ("w2", pc+9));
	registers.insert(pair<string, word_t> ("w3", pc+10));
	registers.insert(pair<string, word_t> ("w4", pc+11));
	registers.insert(pair<string, word_t> ("w5", pc+12));
	registers.insert(pair<string, word_t> ("w6", pc+13));
	registers.insert(pair<string, word_t> ("w7", pc+14));

	registers.insert(pair<string, word_t> ("N0", pc+15)); // not used
	registers.insert(pair<string, word_t> ("N1", pc+16));

	registers.insert(pair<string, word_t> ("R", R)); // return value
	registers.insert(pair<string, word_t> ("R0", pc+17));
	registers.insert(pair<string, word_t> ("R1", pc+18));
	registers.insert(pair<string, word_t> ("R2", pc+19));
	registers.insert(pair<string, word_t> ("R3", pc+20));
	registers.insert(pair<string, word_t> ("R4", pc+21));
	registers.insert(pair<string, word_t> ("R5", pc+22));
	registers.insert(pair<string, word_t> ("R6", pc+23));
	registers.insert(pair<string, word_t> ("R7", pc+24));
	registers.insert(pair<string, word_t> ("SP", sp)); // stack pointer

	registers.insert(pair<string, word_t> ("r", R)); // return value
	registers.insert(pair<string, word_t> ("r0", pc+17));
	registers.insert(pair<string, word_t> ("r1", pc+18));
	registers.insert(pair<string, word_t> ("r2", pc+19));
	registers.insert(pair<string, word_t> ("r3", pc+20));
	registers.insert(pair<string, word_t> ("r4", pc+21));
	registers.insert(pair<string, word_t> ("r5", pc+22));
	registers.insert(pair<string, word_t> ("r6", pc+23));
	registers.insert(pair<string, word_t> ("r7", pc+24));
	registers.insert(pair<string, word_t> ("sp", sp)); // stack pointer

	registers.insert(pair<string, word_t> ("r0l", pc+17));
	registers.insert(pair<string, word_t> ("r1l", pc+18));
	registers.insert(pair<string, word_t> ("r2l", pc+19));
	registers.insert(pair<string, word_t> ("r3l", pc+20));
	registers.insert(pair<string, word_t> ("r4l", pc+21));
	registers.insert(pair<string, word_t> ("r5l", pc+22));
	registers.insert(pair<string, word_t> ("r6l", pc+23));
	registers.insert(pair<string, word_t> ("r7l", pc+24));
	registers.insert(pair<string, word_t> ("spl", sp)); // stack pointer

#	include "display.cpp"
#	include "var.cpp"
#	include "flash.cpp"

}

#ifdef DEBUG
#include <iostream>
#include <iomanip>
#endif

void Opcode::add (string code, string A, string B)
{
	int icode;
	stringstream ss (code);
	if ((ss >> icode).fail())
	{
		// not a number, lets try if its a symbol in the map
		map<string, word_t>::iterator it = commands.find (code);
		if (it == commands.end()) throw invalid_opcode();
		icode = commands[code];
	}

	int iA;
	stringstream sA (A);
	if ((sA >> iA).fail())
	{
		// not a number, lets try if its a symbol in the map
		map<string, word_t>::iterator it = registers.find (A);
		if (it == registers.end()) throw invalid_register('A');
		iA = registers[A];
	}

	int iB;
	stringstream sB (B);
	if ((sB >> iB).fail())
	{
		// not a number, lets try if its a symbol in the map
		map<string, word_t>::iterator it = registers.find (B);
		if (it == registers.end()) throw invalid_register('B');
		iB = registers[B];
	}

	// resizing code (extend for one flash size)
	if (where >= opcodes.size())
	{
#ifdef DEBUG
		using namespace std;
		cout << "Extending" << endl;
#endif
		opcodes.resize(opcodes.size()+128);

	}

	opcodes[where] = op(icode, iA, iB);

#ifdef DEBUG
	using namespace std;
	if (opcodes[where] != 0 && iA != -8192 && iB != -8192) // dont output zeros
	{
		cout
			<< dec << setw(4)
			<< addr(where)
			<< hex
			<< " cur: " << opcodes[where]
			<< " exec: " << icode
			<< " iA: " << iA
			<< " iB: " << iB
			<< endl;
	}
#endif
	where++;
}

char* Opcode::buffer()
{
	return reinterpret_cast<char*>(&opcodes[0]);
}

size_t Opcode::size()
{
	return opcodes.size()*4;
}
