#ifndef CPU_H
#define CPU_H

#include <istream>
#include <vector>
#include <string>
#include <map>

#include "except.h"

class IInstruction;

/**Interface für verschiedene Typen von CPUs.
 * Notwendig für Factory in main.cpp.
 */
class ICPU
{
public:
	virtual void loadProgram (std::istream &in) = 0;
	virtual void saveProgram (std::ostream &out) = 0;

	virtual ~ICPU() {}
};

class CCPU : public ICPU
{
public:
	CCPU ();
	virtual ~CCPU ();
	void loadProgram (std::istream &in);
	void saveProgram (std::ostream &out);
private:
	std::map <std::string, IInstruction*> m_instruction;
	std::vector <IInstruction*> m_program;
	std::map <std::string, int> m_label;
	std::map <std::string, int> m_array;
	size_t m_asm_size;

	void addLabel (std::string const& rest);
	void addArray (std::string const& rest);
};

#endif
