#ifndef INSTANCER_H
#define INSTANCER_H

#include <map>
#include <string>
#include <memory>
#include <stdexcept>

#include "cpu.h"

class Instancer
{
public:
	virtual ICPU* get() = 0;
	virtual ~Instancer() {}
};

class Cnstancer: public Instancer
{
	virtual CCPU* get()
	{
		return new CCPU();
	}
};

class Factory
{
	std::map<std::string, Instancer*> m_factory;
public:
	Factory() :
		m_factory()
	{
		m_factory.insert(std::make_pair("int", new Cnstancer()));
		m_factory.insert(std::make_pair("size_t", new Cnstancer()));
	}

	~Factory()
	{
		for (
			std::map<std::string,Instancer*>::iterator it = 
			m_factory.begin();
			it != m_factory.end();
			it++)
		{
			delete it->second;
		}
	}

	std::auto_ptr<ICPU> get(std::string const& which)
	{
		Instancer* instancer = m_factory[which];
		if (instancer)
		{
			std::auto_ptr <ICPU> ret(instancer->get());
			return ret;
		} else throw std::out_of_range("unknown format");
	}
};

#endif
