#include "getopt_pp.h"

#include "cpu.h"
#include "instancer.h"

// #include "config.h"

#include <fstream>
#include <iostream>
#include <stdexcept>
#include <exception>
#include <memory>

using namespace GetOpt;
using namespace std;

void help();

int main(int argc, char ** argv) try
{
	string infile, outfile;
	// int label = 0;
	//bool needs_calc = false;
	bool needs_help = false;
	GetOpt_pp options (argc, argv);
	options.exceptions(ios::failbit | ios::eofbit);

	options
		// >> Option('l', "label", label)
		>> Option('i', "infile", infile)
		>> Option('o', "outfile", outfile)
		// >> OptionPresent('c', "calc", needs_calc)
		>> OptionPresent('h', "help", needs_help);

	if (needs_help)
	{
		help();
		return 0;
	}

	/*
	if (needs_calc)
	{
		cout << "label  " << label << " is on line " << offset(label)+1 << std::endl;
		return 0;
	}
	*/

	cout 
	     << " Infile: " << infile
	     << " Outfile: " << outfile
	     << " Hilfe: " << needs_help << endl;


	Factory factory;
	auto_ptr<ICPU> cpu = factory.get("int");

	ifstream in (infile.c_str());
	cpu->loadProgram(in);

	ofstream out (outfile.c_str());
	cpu->saveProgram(out);

} catch(GetOptEx const& e)
{
	cerr << argv[0] << " Invalid Options" << endl;
	help();
	return 1;
} catch (out_of_range const& oor)
{
	cerr << argv[0] << " Out of range: " << oor.what() << endl;
	return 2;
} catch (EParse const& p)
{
	cerr << argv[0] << " " << p.what() << " in Program File at line " << p.linenr
		<< ": " << p.msg << endl;
	return 3;
} catch (ifstream::failure const& e)
{
	cerr << argv[0] << " Exception reading file: " << e.what() << endl;
	return 4;
} catch (...)
{
	cerr << argv[0] << " Caught unknown exception" << endl;
	return 100;
}

void help()
{
	cout << "Hilfe zu h8/300-asm" << endl;
	cout << endl;
	cout << "Geben sie mit -i eine H8/300 Assembler Datei zum Lesen an." << endl;
	cout << "Geben sie mit -o eine Basic Assembler Datei zum Schreiben an." << endl;
	cout << endl;
	cout << "Transformiert eine H8/300 Assembler Datei zu" << endl;
	cout << "einer Basic Assember Datei." << endl;
	cout << endl;
	cout << "Unterstützte instructions und directives siehe cpu.cpp." << endl;
	cout << "Parsing siehe cpu.cpp und instruction.cpp" << endl;
	cout << "Die Unterstützten Instruktionen und welcher Code" << endl;
	cout << "generiert wird siehe instruction.h" << endl;
	cout << endl;
}
