#ifndef CONFIG_H
#define CONFIG_H

/**We are simulating a 32 bit CPU.
  Define this type to unsigned 32 bit.
  */
typedef unsigned int word_t;

const int numblock = 32; // number of blocks for all registers

const int regperblock = 128; // number of registers per block
const int blocksize = 512; // size of block in bytes

const int numreg = numblock * blocksize; // number of registers
const int regsize = 4 * numreg; // size of registers in bytes

const int lowaddr = -8192;
const int highaddr = 8191;

const int numregisters = 16384;

const int pc = lowaddr; // programm counter
const int sp = -8168; // stack pointer

const int Z = pc+5;
const int W = pc+6;
const int R = pc+17;

const int start = -8167; // this is used for initial jump

const int lowdisplay = 8064;
const int highdisplay = 8191;

const int lowvar = 8063;
const int highvar = 7936;

const int lowflash = 7808;
const int highflash = 7935;

// the address (0 is lowaddr) from the stack to initialize sp
const int stack = highaddr + lowflash;

const int numaddr = 8192*2;

/**From a register information (lowaddr..highaddr)
  to a positive number for internal opcodes.

  Add 1 to get line information in basicasm files.
  */
inline word_t offset(int number)
{
	return number + 8192;
}

/**Reverses the offset() operation.

  Calculates from an internal 14bit op-code
  to a normal 32 bit signed integer.
 */
inline int addr(word_t internalop)
{
	return internalop - 8192;
}

/* calcs a opcode out of instruction and its parameters.
   */
inline word_t op (int code, int A, int B)
{
	code &= 0x000F;
	A = offset(A);
	B = offset(B);
	A    &= 0x3FFF;
	B    &= 0x3FFF;
	code <<= 28;
	B <<= 14;
	return  code | B | A;
}

/** calcs the instruction and its parameters out of a opcode.

   iA and iB is a number between lowaddr and highaddr.
   It can be used to directly access a register, but also
   to directly calculate with it.
 */
inline void rev (word_t current, word_t& code, int& iA, int& iB)
{
	code = current >> 28;
	code &= 0x000F;

	word_t B = current >> 14;
	B &= 0x3FFF;
	iB = addr(B);

	word_t A = current;
	A &= 0x3FFF;
	iA = addr(A);
}


#define DEBUG
#undef VERBOSE

#endif
