#include "cpu.h"
#include "opcode.h"

#include "register.h"
#include "flash.h"
#include "display.h"
#include "keyboard.h"

CPU::CPU(Register &reg, Flash &flash, Display &dis,
			Keyboard &key) :
	opcodes(),
	reg(reg),
	flash(flash),
	dis(dis),
	key(key)
{
	opcodes.push_back(new SetConstantOpcode(reg)); // 0
	opcodes.push_back(new SetOpcode(reg)); // 1
	opcodes.push_back(new SetAddressConstantOpcode(reg)); // 2
	opcodes.push_back(new SetAddressOpcode(reg)); // 3
	opcodes.push_back(new AddConstantOpcode(reg)); // 4
	opcodes.push_back(new AddOpcode(reg)); // 5
	opcodes.push_back(new AddAddressOpcode(reg)); // 6
	opcodes.push_back(new SubOpcode(reg)); // 7
	opcodes.push_back(new MulConstantOpcode(reg)); // 8
	opcodes.push_back(new MulOpcode(reg)); // 9
	opcodes.push_back(new ModOpcode(reg)); // A
	opcodes.push_back(new DivOpcode(reg)); // B
	opcodes.push_back(new JumpOpcode(reg)); // C
	opcodes.push_back(new KeyOpcode(reg, key, dis)); // D
	opcodes.push_back(new WriteOpcode(reg, flash)); // E
	opcodes.push_back(new ReadOpcode(reg, flash)); // F
}

CPU::~CPU()
{
	for (size_t i=0; i<opcodes.size(); i++)
	{
		delete opcodes[i];
	}
}

#ifdef DEBUG
#include <iostream>
#include <iomanip>
#endif

void CPU::exec()
{
	word_t current = reg[reg[pc]];
	word_t code;
	int iA, iB;
	rev (current, code, iA, iB);

#ifdef DEBUG
	using namespace std;
	cout
		<< dec << setw(4)
		<< reg[pc]
		// << " cur: " << current
		<< " exec: " << opcodes[code]->name()
		<< " iA: " << iA
		<< " iB: " << iB
		<< " sp: " << reg[sp]
		<< " *sp: " << reg[reg[sp]]
		<< " Z: " << reg[Z]
		<< " W: " << reg[W]
		<< " R: " << reg[R]
		<< " line: " << offset(reg[pc])+1 // because line start with 1
		<< endl;
#endif

	(*opcodes[code])(iA,iB);
}

