#include "flash.h"

#include <string>

#include "register.h"
#include "config.h"

#ifdef DEBUG
#include <iostream>
#include <iomanip>
#endif

using namespace std;

Flash::Flash(string const& file) :
	f(file.c_str(), ios::in | ios::out | ios::binary)
{
	if (!f.is_open()) throw file_error (file.c_str());
	f.exceptions(ifstream::eofbit | ifstream::failbit |
			ifstream::badbit);
}

Flash::~Flash()
{}

void Flash::writeRegister(Register &reg)
{
	word_t block [numreg];
	f.seekg(0);
	f.read(reinterpret_cast<char *> (block), regsize);
	if (!f.good()) throw file_error("could not read flash for initialization");
	for (int i=lowaddr; i<=highaddr; i++)
	{
		reg[i] = block[offset(i)];
	}
}

void Flash::writeBlock(Register &reg, size_t block, int from)
{
#ifdef DEBUG
	using namespace std;
	cout << "write flash block: " << dec << block
		<< " from reg: " << from << endl;
#endif
	f.seekg(block * blocksize);
	f.write(reg.buffer(from), blocksize);
}

void Flash::readBlock(Register &reg, size_t block, int to)
{
#ifdef DEBUG
	using namespace std;
	cout << "read flash block: " << dec << block
		<< " to reg: " << to << endl;
#endif
	f.seekg(block * blocksize);
	f.read(reg.buffer(to), blocksize);
}
